function [H, theta, MuX, MuY, sX, sY, llh, BIC] = ...
    gmm(X, Y, K, maxIter, replicates, verbose)
%GMM trains a Gaussian mixture model for gene and miRNA expression data.
%
%
%   
%
% References:
% [1] Bahadir Ozdemir, Wael Abd-Almageed, Stephanie Roessler, Xin Wei Wang.
%     "iSubgraph: Integrative Genomics for Subgroup Discovery in
%     Hepatocellular Carcinoma using Graph Mining and Mixture Models",
%     PLoS ONE (under review).
%
%
%
%   Copyright (C) 2013 Bahadir Ozdemir  
%
%   This file is part of iSubgraph.
%
%   iSubgraph is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%    iSubgraph is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.


llh = -inf;
for i = 1:replicates
    [H2,theta2,MuX2,MuY2,sX2,sY2,llh2,BIC2] = gmm2(X, Y, K, maxIter, verbose);
    if llh2 > llh
        H = H2;
        theta = theta2;
        MuX = MuX2;
        MuY = MuY2;
        sX = sX2;
        sY = sY2;
        llh = llh2;
        BIC = BIC2;
    end
end


end



function [H,theta,MuX,MuY,sX,sY,llh,BIC] = gmm2(X, Y, K, maxIter, verbose)

G = size(X,1);
[M,N] = size(Y);

H = init(X,Y,K);
for iter = 1:maxIter
    theta = mean(H,2);
    
    MuX = zeros(G,K);
    tmp = 0;
    for i = 1:G
        valid = ~isnan(X(i,:));
        for k = 1:K
            MuX(i,k) = (X(i,valid) * H(k,valid)') / (sum(H(k,valid))+eps);
            tmp = tmp + ((X(i,valid) - MuX(i,k)).^2) * H(k,valid)';
        end
    end
    sX = sqrt(tmp / sum(sum(~isnan(X))));
    
    MuY = zeros(M,K);
    tmp = 0;
    for j = 1:M
        valid = ~isnan(Y(j,:));
        for k = 1:K
            MuY(j,k) = (Y(j,valid) * H(k,valid)') / (sum(H(k,valid))+eps);
            tmp = tmp + ((Y(j,valid) - MuY(j,k)).^2) * H(k,valid)';
        end
    end
    sY = sqrt(tmp / sum(sum(~isnan(Y))));
    
    H0 = H;
    [H,llh] = predict(X, Y, theta, MuX, MuY, sX, sY);
    
    diff = norm(H0-H,'fro') / numel(H);
    if verbose
        fprintf(1,'%d\t%.2e\t%.2e\n', iter, diff, llh);
    end
    
    if diff < 10^-6
        if verbose
            fprintf(1,'===\n\n');
        end
        break;
    end
end

nParams = K*(M+G+1) + 2;
BIC = -2*llh + nParams*log(N);
end




function H = init(X,Y,K)

N = size(X,2);
idx = randsample(N,K);

X = knnimpute([X;Y], 3);
Mu = X(:,idx);
[~,label] = max(bsxfun(@minus, Mu'*X, dot(Mu,Mu,1)'/2), [], 1);
[u,~,label] = unique(label);
while K ~= length(u)
    idx = randsample(N,K);
    Mu = X(:,idx);
    [~,label] = max(bsxfun(@minus, Mu'*X, dot(Mu,Mu,1)'/2), [], 1);
    [u,~,label] = unique(label);
end
H = full(sparse(1:N, label, 1, N, K, N));
[~,label] = max(H, [], 2);
H = H(:,unique(label))';
end




function [H, llh] = predict(X, Y, theta, MuX, MuY, sX, sY)
K = size(MuX,2);
N = size(X,2);

logProb = zeros(K,N);
for k = 1:K
    logProb(k,:) = log(theta(k)) + logGauss(X, MuX(:,k), sX) + logGauss(Y, MuY(:,k), sY);
end

largests = max(logProb,[],1);
tmp = bsxfun(@minus, logProb, largests);
total = largests + log(sum(exp(tmp)));
idx = find(~isfinite(total));
if ~isempty(idx)
    total(idx) = largests(idx);
end

llh = sum(total);
H = exp(bsxfun(@minus, logProb, total));
end




function logProb = logGauss(X, mu, s)
N = size(X,2);
logProb = zeros(1,N);

if any(X)
    for n = 1:N
        valid = ~isnan(X(:,n));
        logProb(n) = -sum(valid)*log(2*pi)/2 - sum(valid)*log(s) ...
            - sum(((X(valid,n)-mu(valid))/s).^2)/2;
    end
end
end
