function graphWriter(gene_expr, miRNA_expr, thresh, targets)
%GRAPHWRITER constructs patient graphs and writes them to a file as input to the
%MoSS software by Christian Borgelt.
%
%   GRAPHWRITER(GENE_EXPR, MIRNA_EXPR, THRESH, TARGETS) constructs patient
%   graphs for each patient based on the template bipartite graph defined by
%   miRNA-gene target relationships and writes all patient graphs to a file
%   'patients.g', as input to subsequent graph mining software MoSS. GENE_EXPR
%   is a GxN matrix that includes gene expression from tumor tissues only where
%   columns correspond patients and rows correspond
%   genes. Similarly, MIRNA_EXPR_T is a MxN matrix that includes gene expression
%   from tumor tissues only where columns correspond patients and rows
%   correspond miRNAs. THRESH is the threshold used for UP and DOWN tags. Use 1
%   for thresholds UP:+1 / DOWN:-1. TARGETS is a GxM matrix that each element
%   indicates correlated target relationship between corresponding gene (row)
%   and miRNA (column). The values must be +1 for positively correlated targets,
%   -1 for negatively correlated targets and 0 for non-targets.
%
%
%
% References:
% [1] Bahadir Ozdemir, Wael Abd-Almageed, Stephanie Roessler, Xin Wei Wang.
%     "iSubgraph: Integrative Genomics for Subgroup Discovery in
%     Hepatocellular Carcinoma using Graph Mining and Mixture Models",
%     PLoS ONE (under review).
% [2] Mathias Fiedler and Christian Borgelt. "Support Computation for Mining
%     Frequent Subgraphs in a Single Graph", Proc. 5th Int. Workshop on Mining
%     and Learning with Graphs (MLG 2007, Florence, Italy).
%
%
%
%   Copyright (C) 2013 Bahadir Ozdemir
%
%   This file is part of iSubgraph.
%
%   iSubgraph is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%    iSubgraph is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.


% size check
[G,M] = size(targets);
[GG,N] = size(gene_expr);
[MM,NM] = size(miRNA_expr);
if (N ~= NM) || (G ~= GG) || (M ~= MM)
    error('iSubgraph: matrix sizes dont match');
end


% open file and organize target correlation type for each miRNA
fid = fopen('patients.g', 'w');

pos_targets = cell(M,1);
neg_targets = cell(M,1);
for i = 1:M
    pos_targets{i} = find(targets(:,i) == 1);
    neg_targets{i} = find(targets(:,i) == -1);
end


% write to file
count = 0;
for i = 1:N
    count = count + singleSampleWriter(fid, i, gene_expr(:,i), ...
        miRNA_expr(:,i), thresh, pos_targets, neg_targets);
    disp(['Patient graph ', num2str(i), ' has been written to file.']);
end
disp(['In total, ', num2str(count), ' patient graphs have been written.']);
fclose(fid);




function [result] = singleSampleWriter(fid, sampleId, genes, miRNAs, thresh, ...
    pos_targets, neg_targets)
% writes each individual patient graph to file

% determine node labels
gene_idx = zeros(size(genes));
gene_up = genes >= thresh;
gene_down = genes <= -thresh;

miRNA_idx = zeros(size(miRNAs));
miRNA_up = miRNAs >= thresh;
miRNA_down = miRNAs <= -thresh;

if sum(gene_up + gene_down) == 0 ||  sum(miRNA_up + miRNA_down) == 0
    result = 0;
    return;
end

% write nodes
count = 1;
for i = find(gene_up)'
    fprintf(fid, 'n %d g%dp\n', count, i);
    gene_idx(i) = count;
    count = count + 1;
end

for i = find(gene_down)'
    fprintf(fid, 'n %d g%dn\n', count, i);
    gene_idx(i) = -count;
    count = count + 1;
end

for i = find(miRNA_up)'
    fprintf(fid, 'n %d m%dp\n', count, i);
    miRNA_idx(i) = count;
    count = count + 1;
end

for i = find(miRNA_down)'
    fprintf(fid, 'n %d m%dn\n', count, i);
    miRNA_idx(i) = -count;
    count = count + 1;
end

% write edges
for i = find(miRNA_idx > 0)'
    for j = find(gene_idx(pos_targets{i}) > 0)'
        fprintf(fid, 'e %d %d ed\n', miRNA_idx(i), ...
            gene_idx(pos_targets{i}(j)));
    end
end

for i = find(miRNA_idx > 0)'
    for j = find(gene_idx(neg_targets{i}) < 0)'
        fprintf(fid, 'e %d %d ed\n', miRNA_idx(i), ...
            -gene_idx(neg_targets{i}(j)));
    end
end

for i = find(miRNA_idx < 0)'
    for j = find(gene_idx(neg_targets{i}) > 0)'
        fprintf(fid, 'e %d %d ed\n', -miRNA_idx(i), ...
            gene_idx(neg_targets{i}(j)));
    end
end

for i = find(miRNA_idx < 0)'
    for j = find(gene_idx(pos_targets{i}) < 0)'
        fprintf(fid, 'e %d %d ed\n', -miRNA_idx(i), ...
            -gene_idx(pos_targets{i}(j)));
    end
end

fprintf(fid, 'g Graph %d\n', sampleId);
fprintf(fid, 'x 0\n\n');
result = 1;
