function [cls] = main(gene_expr_t, gene_expr_nt, miRNA_expr_t, miRNA_expr_nt, ...
    targets, z_thresh, min_node, support, k)
%MAIN demonstrates how to use the iSubgraph algorithm. Users should enter
%support threshold to the textbox, "Minimal support in focus" in the 3rd tab of
%MoSS software.
%
%   CLS = MAIN(GENE_EXPR_T, GENE_EXPR_NT, MIRNA_EXPR_T, MIRNA_EXPR_NT, TARGETS,
%   Z_THRESH, MIN_NODE, SUPPORT, K) combines all parts of the iSubgraph
%   algorithm. GENE_EXPR_T is a GxN matrix that includes gene expression from
%   tumor tissues only where columns correspond patients and rows correspond
%   genes. GENE_EXPR_NT includes gene expression from nontumor tissues only.
%   Similarly, MIRNA_EXPR_T is a MxN matrix that includes gene expression from
%   tumor tissues only where columns correspond patients and rows correspond
%   miRNAs. MIRNA_EXPR_NT includes gene expression from nontumor tissues only.
%   TARGETS is a GxM matrix that each element indicates target relationship
%   between corresponding gene (row) and miRNA (column). The values must be 1
%   for targets and 0 for non-targets. Z_THRESH is the threshold used for UP and
%   DOWN tags. Use 1 for thresholds UP:+1 / DOWN:-1. MIN_NODE is the minimum
%   number of nodes in closed frequent subgraphs that will be used in patient
%   clustering. SUPPORT is the minimum number of patients that must include a
%   "frequent" subgraph. K is the number of tumor subgroups. CLS is a 1xN vector
%   that includes class predictions for all patients.
%
%
% References:
% [1] Bahadir Ozdemir, Wael Abd-Almageed, Stephanie Roessler, Xin Wei Wang.
%     "iSubgraph: Integrative Genomics for Subgroup Discovery in
%     Hepatocellular Carcinoma using Graph Mining and Mixture Models",
%     PLoS ONE (under review).
% [2] Mathias Fiedler and Christian Borgelt. "Support Computation for Mining
%     Frequent Subgraphs in a Single Graph", Proc. 5th Int. Workshop on Mining
%     and Learning with Graphs (MLG 2007, Florence, Italy).
%
%
%   Copyright (C) 2013 Bahadir Ozdemir
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.


% size check
[G,M] = size(targets);
[GG,N] = size(gene_expr_t);
[MM,NM] = size(miRNA_expr_t);
if (N ~= NM) || (G ~= GG) || (M ~= MM) || ...
        (G ~= size(gene_expr_nt,1)) || (M ~= size(miRNA_expr_nt,1))
    error('iSubgraph: matrix sizes dont match');
end

% combine expression profiles
N = size(gene_expr_t,2);
genes = [gene_expr_t, gene_expr_nt];
miRNAs = [miRNA_expr_t, miRNA_expr_nt];

% determine actual targets
[corr_targets] = targetPredictor(genes, miRNAs, targets);

% convert expression levels to z-scores
tmp = bsxfun(@rdivide, bsxfun(@minus, genes, nanmean(genes,2)), ...
    nanstd(genes,[],2));
X = tmp(:,1:N);
tmp = bsxfun(@rdivide, bsxfun(@minus, miRNAs, nanmean(miRNAs,2)), ...
    nanstd(miRNAs,[],2));
Y = tmp(:,1:N);

% remove unnecassary nodes from patient graphs
X(sum(~~targets,2)==0,:) = 0;
Y(sum(~~targets)==0,:) = 0;

% write patient graphs to files
graphWriter(X, Y, z_thresh, corr_targets);

% Subgraph support should be entered in 3rd tab "Minimal support in focus" part.
system('java -jar -Xms256m -Xmx512m -Xss128m ./moss.jar ./moss.conf');

% read and combine subgraphs
[patterns, gene_count] = readSubgraph('patterns', X, Y);
module = combineSubgraphs(patterns(gene_count >= min_node));

% patient clustering via mixture model
selected_genes = [module.gene_up; module.gene_down];
selected_miRNAs = [module.miRNA_up; module.miRNA_down];
H = gmm(gene_expr_t(selected_genes,:), miRNA_expr_t(selected_miRNAs,:), ...
    k, 300, 10, 0);
[~,cls] = max(H);
