function [patterns, gene_count, miRNA_count] = readSubgraph(filename, genes, miRNAs)
%READSUBGRAPHS reads the output of MoSS for the closed frequent subgraphs.
%
%   [PATTERNS, GENE_COUNT, MIRNA_COUNT] = READSUBGRAPHS(FILENAME, GENES, MIRNAS)
%   read the output of MoSS software named FILENAME, GENES contains the gene
%   expression data including G rows, and MIRNAS contains miRNA expression data
%   including M rows. PATTERNS is a structure array including all closed
%   frequent subgraphs, GENE_COUNT is the number of gene nodes of the
%   corresponding subgraphs and MIRNA_COUNT is the number of miRNA nodes of the
%   corresponding subrgaphs.
%
%
% References:
% [1] Bahadir Ozdemir, Wael Abd-Almageed, Stephanie Roessler, Xin Wei Wang.
%     "iSubgraph: Integrative Genomics for Subgroup Discovery in
%     Hepatocellular Carcinoma using Graph Mining and Mixture Models",
%     PLoS ONE (under review).
% [2] Mathias Fiedler and Christian Borgelt. "Support Computation for Mining
%     Frequent Subgraphs in a Single Graph", Proc. 5th Int. Workshop on Mining
%     and Learning with Graphs (MLG 2007, Florence, Italy).
%
%
%
%   Copyright (C) 2013 Bahadir Ozdemir
%
%   This file is part of iSubgraph.
%
%   iSubgraph is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%    iSubgraph is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.


% open files
G = size(genes,1);
M = size(miRNAs,1);
fid = fopen(strcat(filename, '.g'), 'r');
count = 1;
while ~feof(fid)
    gene_up = zeros(G,1);
    gene_down = zeros(G,1);
    miRNA_up = zeros(M,1);
    miRNA_down = zeros(M,1);
    
    line = fgetl(fid);
    tmp = strread(line, '%s');
    while ~isempty(tmp)
        switch tmp{1}
            case 'n'
                idx = str2double(tmp{3}(2:end-1));
                switch strcat(tmp{3}(1), tmp{3}(end))
                    case 'gp'
                        gene_up(idx) = 1;
                    case 'gn'
                        gene_down(idx) = 1;
                    case 'mp'
                        miRNA_up(idx) = 1;
                    case 'mn'
                        miRNA_down(idx) = 1;
                end
            case 's'
                patterns(count).gene_up = find(gene_up);
                patterns(count).gene_down = find(gene_down);
                patterns(count).miRNA_up = find(miRNA_up);
                patterns(count).miRNA_down = find(miRNA_down);
                patterns(count).size = str2double(tmp{2});
                patterns(count).freq = str2double(tmp{4});
                count = count + 1;
        end
        line = fgetl(fid);
        tmp = strread(line, '%s');
    end
    
    if mod(count,1000) == 0
        disp([num2str(count), ' subgraphs has been read.']);
    end
end
disp(['In total, ', num2str(count), ' subgraphs has been found.' ...
    ' Duplicates subgraphs will be removed.']);
fclose(fid);

% remove duplicate subgraphs
n = length(patterns);
pattern_matrix = sparse(n, G+M);
for i = 1:n
    pattern_matrix(i, patterns(i).gene_up) = 1;
    pattern_matrix(i, patterns(i).gene_down) = -1;
    pattern_matrix(i, patterns(i).miRNA_up + G) = 1;
    pattern_matrix(i, patterns(i).miRNA_down + G) = -1;
end

[~,idx,~] = unique(pattern_matrix, 'rows');
patterns = patterns(idx);
n = length(patterns);
gene_count = zeros(n,1);
miRNA_count = zeros(n,1);
for i = 1:n
    if isempty(patterns(i).gene_up)
        patterns(i).gene_up = [];
    else
        patterns(i).gene_up = patterns(i).gene_up(:);
    end
    
    if isempty(patterns(i).gene_down)
        patterns(i).gene_down = [];
    else
        patterns(i).gene_down = patterns(i).gene_down(:);
    end
    
    if isempty(patterns(i).miRNA_up)
        patterns(i).miRNA_up = [];
    else
        patterns(i).miRNA_up = patterns(i).miRNA_up(:);
    end
    
    if isempty(patterns(i).miRNA_down)
        patterns(i).miRNA_down = [];
    else
        patterns(i).miRNA_down = patterns(i).miRNA_down(:);
    end
    
    gene_count(i) = length(patterns(i).gene_up) + ...
        length(patterns(i).gene_down);
    miRNA_count(i) = length(patterns(i).miRNA_up) + ...
        length(patterns(i).miRNA_down);
end
