function [corr_targets] = targetPredictor(gene_expr, miRNA_expr, targets)
%TARGETPREDICTOR identifies correlated target genes of each miRNA from
%expression data.
%
%   CORR_TARGETS = GRAPHWRITER(GENE_EXPR, MIRNA_EXPR, TARGETS) identifies the
%   actual target genes of each miRNA using correlation analysis. GENE_EXPR is
%   GxN matrices that includes gene expression values from both tumor and
%   nontumor tissues. The columns correspond patients and rows correspond genes.
%   Similarly, MIRNA_EXPR is MxN matrices that includes miRNA expression values
%   from both tumor and nontumor tissues. TARGETS is a GxM binary matrix that
%   indicates target genes of miRNAs from a sequence based analysis where 1 for
%   targets and 0 for non-targets. The columns correspond miRNAs and rows
%   correspond genes.
%
%
% References:
% [1] Bahadir Ozdemir, Wael Abd-Almageed, Stephanie Roessler, Xin Wei Wang.
%     "iSubgraph: Integrative Genomics for Subgroup Discovery in
%     Hepatocellular Carcinoma using Graph Mining and Mixture Models",
%     PLoS ONE (under review).
%
%
%
%   Copyright (C) 2013 Bahadir Ozdemir
%
%   This file is part of iSubgraph.
%
%   iSubgraph is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%    iSubgraph is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.


% size check
[G,M] = size(targets);
[GG,N] = size(gene_expr);
[MM,NM] = size(miRNA_expr);
if (N ~= NM) || (G ~= GG) || (M ~= MM)
    error('iSubgraph: matrix sizes dont match');
end

% compute Pearson correlation
corr = 1 - pdist2(gene_expr, miRNA_expr, 'correlation');
corr_type = (corr > 0) - (corr < 0);

% permute, recalculate Pearson correlation and count significant cases
count = zeros(G,M);
num_tries = 1000; % can be changed to 10000
for i = 1:num_tries
    tmp_gene = gene_expr(:,randperm(N));
    tmp_miRNA = miRNA_expr(:,randperm(N));
    tmp_corr = zeros(G,M);
    for j = 1:M
        tmp_corr(targets(:,j), j) = 1 - pdist2(tmp_gene(targets(:,j),:), ...
            tmp_miRNA(j,:), 'correlation');
    end
    count = count + ((tmp_corr >= corr) & (corr_type == 1)) + ...
        ((tmp_corr <= corr) & (corr_type == -1));
    
    if mod(i,10) == 0
        disp(['Progress: ', num2str(i/10), '/100']);
    end
end

% compute FDR for each miRNA and decide actual targets
corr_targets = zeros(G,M);
for i = 1:M
    p = sort(count(targets(:,i),i) / num_tries);
    tmp = find(p(:)' <= 0.05*(1:length(p))/length(p), 1, 'last');
    if ~isempty(tmp)
        corr_targets(targets(:,i),i) = corr_type(targets(:,i),i) .* ...
            count(targets(:,i),i)/1000 <= p(tmp);
    end
end
